(*****************************************************************************
 *                                                                           *
 *      Interface graphique : adaptation de la bibliothque libgraph         *
 *                                                                           *
 *****************************************************************************)

(* Michel Quercia, le 10 aot 1997 *)

type point = {x:int; y:int};;
(* Toutes les spcifications de points sont effectues  l'aide de ce type. *)

(*
 * Les dfinitions suivantes remplacent celles du module [graphics],
 * voir [graphics.mli] ou le manuel de rfrence pour leurs descriptions.
 *)

value open_graph  : string -> unit
  and close_graph : unit -> unit
  and clear_graph : unit -> unit
  and size_x : unit -> int
  and size_y : unit -> int
;;

type color == int;;
value rgb: int -> int -> int -> color
  and set_color : color -> unit
  and black   : color
  and white   : color
  and red     : color
  and green   : color
  and blue    : color
  and yellow  : color
  and cyan    : color
  and magenta : color
  and background : color
  and foreground : color
;;

value plot           : point -> unit
  and point_color    : point -> color
  and moveto         : point -> unit
  and current_point  : unit  -> point
  and lineto         : point -> unit
  and draw_arc       : point -> int -> int -> int -> int -> unit
  and draw_ellipse   : point -> int -> int -> unit
  and draw_circle    : point -> int -> unit
  and set_line_width : int -> unit
  and draw_char      : char -> unit
  and draw_string    : string -> unit
  and set_font       : string -> unit
  and set_text_size  : int -> unit
  and text_size      : string -> int * int
  and fill_rect      : point -> int -> int -> unit
  and fill_poly      : point vect -> unit
  and fill_arc       : point -> int -> int -> int -> int -> unit
  and fill_ellipse   : point -> int -> int -> unit
  and fill_circle    : point -> int -> unit
;;

type status = {
  mouse      : point;         (* position de la souris                      *)
  button     : bool;          (* true si un bouton est enfonc              *)
  keypressed : bool;          (* true si une nouvelle touche est disponible *)
  key        : char }         (* le cractrre frapp                        *)

  and event =
  | Button_down               (* Appui sur un bouton                        *)
  | Button_up                 (* Relchement d'un bouton                    *)
  | Key_pressed               (* Appui sur une touche                       *)
  | Mouse_motion              (* Mouvement de la souris                     *)
  | Poll                      (* Ne pas attendre, retourner immdiatement   *)
;;

value wait_next_event : event list -> status
  and mouse_pos       : unit -> point
  and button_down     : unit -> bool
  and read_key        : unit -> char
  and key_pressed     : unit -> bool
  and sound           : int -> int -> unit
;;

(*
 * Les dfinitions suivantes compltent les primitives de trac fournies par
 * [libgraph].
 *)
value curveto        : point -> point -> point -> unit
  and draw_poly      : point vect -> unit
  and poly_of_bezier : int -> point vect -> point vect
  and draw_bezier    : int -> point vect -> unit
  and fill_bezier    : int -> point vect -> unit
;;
(*
 * [curveto b c d] trace une approximation de l'arc cubique de Bzier dfini
 *    par le point courant, les points de contrle [b] et [c] et le point final
 *    [d]. Le point courant est plac en [d]  la fin du trac.
 *
 * [draw_poly v] trace l'arc polygonal passant par les points de [v]. Le point
 *    courant est plac  la fin de [v].
 *
 * [poly_of_bezier d v] retourne un polygone (vecteur de points) approchant
 *    l'arc cubique par morceaux de Bzier associ aux points de [v].
 *    L'arc numro [n] a pour point de dpart [v.(3n)], pour points de contrle
 *    [v.(3n+1)] et [v.(3n+2)], et pour point final [v.(3n+3)]. La longueur de
 *    [v] doit tre un multiple de 3 plus un, sinon l'exception
 *    [Invalid_argument "poly_of_bezier"] est gnre.
 *    Le paramtre [d] contrle la qualit de l'approximation : chaque arc est
 *    divis rcursivement [d] fois de suite en deux sous-arcs par la mthode
 *    des milieux, le polygone retourn est celui constitu des points
 *    obtenus (contrle et extrmits). La primitive [curveto] utilise cet
 *    algorithme avec [d=4].
 *
 * [draw_bezier d v] et [fill_bezier d v] sont fonctionnellement quivalents 
 *    [draw_poly (poly_of_bezier d v)] et [fill_poly (poly_of_bezier d v)].
 *)

                       (* +-----------------+
                          |  Images bitmap  |
                          +-----------------+ *)


type image == graphics__image;;
value transp        : color
  and make_image    : color vect vect -> image
  and dump_image    : image -> color vect vect
  and draw_image    : image -> int -> int -> unit
  and get_image     : int   -> int -> int -> int -> image
  and create_image  : int   -> int -> image
  and blit_image    : image -> int -> int -> unit
  and current_image : unit -> color vect vect
;;

                         (* +-----------------------+
                            |  Images vectorielles  |
                            +-----------------------+ *)

type graphic_cmd =
| Set_color      of color
| Set_line_width of int
| Set_font       of string
| Set_text_size  of int
| Poly           of bool * (point vect)
| Bezier         of bool * int * (point vect)
| Arc            of bool * point * int * int * int * int
| Ellipse        of bool * point * int * int
| String         of point * string
;;
(*
 * commandes reconnues par la bibliothque
 *
 * le paramtre boolen pour [Poly], [Bezier], [Arc] et [Ellipse] indique
 * si le trac doit tre rempli ou non.
 *)

type vimage == graphic_cmd list;;
(* liste des commandes produisant une image *)

value set_record     : bool -> unit   (* active/dsactive l'enregistrement des commandes *)
  and current_vimage : unit -> vimage (* image actuellement enregistre *)
  and display_vimage : vimage -> unit (* affiche [img] sans effacer l'cran *)
;;
(*
 * [set_record true]  active l'enregistrement des commandes graphiques,
 *                    initialement l'enregistrement est activ.
 * [set_record false] le dsactive, la liste des commandes enregistres n'est
 *                    pas modifie.
 * [current_vimage()] retourne une copie des commandes enregistres.
 * [display_vimage img] excute les commandes figurant dans la liste [img].
 *)

type bounding_box = VIDE | BB of bb
and bb = {xmin:int; xmax:int; ymin:int; ymax:int};;

value bounding_box : vimage -> bounding_box;;
(* Rectangle d'encombrement d'une image *)
(* les textes sont compts avec une hauteur de 20 pixels et une largeur   *)
(* de 10 pixels par caractre. Cette estimation forfaitaire peut conduire *)
(*  une bounding-box incorrecte.                                         *)

                   (* +------------------------+
                      |  Conversion de format  |
                      +------------------------+ *)

value output_postscript : bool -> out_channel -> vimage -> unit
  and output_fig        : bool -> out_channel -> vimage -> unit
  and output_pstricks   : bool -> out_channel -> vimage -> unit
  and output_tga        : out_channel -> color vect vect -> unit
  and output_ppm        : out_channel -> color vect vect -> unit
  and output_ppmraw     : out_channel -> color vect vect -> unit

  and write_postscript  : string -> unit
  and write_fig         : string -> unit
  and write_pstricks    : string -> unit
  and write_tga         : string -> unit
  and write_ppm         : string -> unit
  and write_ppmraw      : string -> unit
;;

(*
 [output_xxx [b] canal img] enregistre le code [xxx] de l'image [img] sur le
 canal de sortie [canal]. Le boolen [b] indique si les instructions
 originales doivent tre incorpores en commentaire dans  le code produit.

 [write_xxx nom] enregistre le code [xxx] de l'image courante dans le
 le fichier de nom [nom]. Pour les formats vectoriels, les instructions
 originales sont incorpores en commentaire dans  le code produit.

 formats vectoriels :
 
 xxx = postscript => postscript encapsul
 xxx = fig        => FIG-3.2
 xxx = pstricks   => macros pstricks

 formats ponctuels :

 xxx = tga        => targa 24 bits non compress
 xxx = ppm        => PPM 24 bits cod ascii
 xxx = ppmraw     => PPM 24 bits cod binaire

 Limitations :

 1. Les changements de fonte et de taille de texte sont ignors dans les
    formats vectoriels. La fonte utilise est la fonte Helvetica avec une
    taille de 10 points pour [output_postscript], la fonte par dfaut pour
    [output_fig] et [output_pstricks].

 2. Le format FIG-3.2 ne reconnait plus les cubiques de Bzier. La fonction
    [output_fig] gnre  la place des "x-splines" avec 4 fois plus de points
    de contrle de faon  limiter la dissemblance avec les courbes de
    Bzier ordinaires.

Mise  l'chelle :

[output_postscript] produit du code postscript encapsul, la taille de
l'image est spcifie dans le commentaire "%%BoundingBox". Cette taille
peut tre change par modification de la matrice de transformation courante.

[output_fig] produit une image  100 dpi. Modifier au besoin la rsolution
indique (ligne 9 du fichier produit).

[output_pstricks] produit une image  1 pixel/point. Modifier au besoin la
valeur du paramtre "unit", ligne 5 du fichier produit.
*)

