(*******************************************************************************
 *                                                                             *
 *   Graphx : conversion vimage -> pstricks                                    *
 *                                                                             *
 *******************************************************************************)

open Graph;;
open Printf;;
open Utils;;

(* sort le code pstricks de [img] sur le canal [chout] *)
(* [comm] indique s'il faut placer en commentaire les commandes originales *)

let output_pstricks comm chout img =

  let emit format = fprintf chout format
  and emitp format p = fprintf chout format p.x p.y in
  let emitv format v =
    for i=0 to Array.length v - 1 do emitp format v.(i) done;
    emit "\n"
  and comment = (if comm then comment chout "%" else fun _ -> ())
  in

  (* dnomination des couleurs *)
  let color = Hashtbl.create 17 in
  Hashtbl.add color black   "black";	
  Hashtbl.add color white   "white";	
  Hashtbl.add color red     "red";	
  Hashtbl.add color green   "green";	
  Hashtbl.add color blue    "blue";	
  Hashtbl.add color yellow  "yellow";	
  Hashtbl.add color cyan    "cyan";	
  Hashtbl.add color magenta "magenta";
  let colnum = ref 0 in
  let coul(c) =
    try Hashtbl.find color c
    with Not_found ->
      incr colnum;
      let nom = (sprintf "color-%d" !colnum) in
      Hashtbl.add color c nom;
      nom
  in                      

  (* polygone *)
  let poly(fill,v) =
    emit "\\psline%s{-}" (if fill then "*" else "");
    emitv "(%d,%d)" v
  in

  (* courbe de Bzier *)
  let bezier(fill,v) = 
    emit "\\pscustom%s{\n" (if fill then "*" else "");
    let i = ref 0 in
    while !i+3 < Array.length v do
      emitp "\\psbezier{-}(%d,%d)" v.(!i);
      emitp "(%d,%d)"   v.(!i+1);
      emitp "(%d,%d)"   v.(!i+2);
      emitp "(%d,%d)\n" v.(!i+3);
      i := !i + 3 
    done;
    emit "}\n"
  in

  (* chaine de caractres *)
  (* il faut chapper tous les caractres spciaux de TeX ... *)
  let string(p,s) =
    emitp "\\rput[Bl]{*0}(%d,%d){" p;
    for i=0 to String.length s - 1 do match s.[i] with
    | ('{'|'}'|'$'|'&'|'#'|'^'|'_'|'%'|'~' as c) -> emit "\\%c" c
    | '\\' -> emit "$\\backslash$"
    | c -> emit "%c" c
    done;
    emit "}\n"
  in

  (* fonction gnrale de conversion *)
  let pstricks(cmd) =
    comment cmd;
    match cmd with

    | Set_color(c) -> 
      let oldnum = !colnum in
      let co = coul(c)     in
      if !colnum > oldnum then  begin
        let r = float(c lsr 16)/.255.0
        and g = float((c lsr 8) land 255)/.255.0
        and b = float(c land 255)/.255.0 in
        emit "\\newrgbcolor{%s}{%f %f %f}" co r g b
      end;
      emit "\\psset{linecolor=%s}\\csname %s\\endcsname\n" co co
 
    | Set_line_width(w) -> emit "\\psset{linewidth=%d}\n" w
    | Set_font(f) -> ()
    | Set_text_size(sz) -> ()

    | Poly(fill,v) -> poly(fill,v)
    | Bezier(fill,d,v) ->
      if d >= 4 then bezier(fill,v) else poly(fill,poly_of_bezier d v)
    | Arc(fill,p,rx,ry,a1,a2) -> bezier(fill,bezier_of_arc(fill,p,rx,ry,a1,a2))
    | Ellipse(fill,p,rx,ry)   -> bezier(fill,bezier_of_arc(false,p,rx,ry,0,360))
    | String(p,s) -> string(p,s)
  in

  (* prologue *)
  emit "\\input pstricks\n\\nopagenumbers\n\\overfullrule=0pt\n\n";
  emit "\\psset{unit=1pt}\n";
  begin match bounding_box img with
  |BB(bb) -> emit "\\pspicture(%d,%d)(%d,%d)\n" bb.xmin bb.ymin bb.xmax bb.ymax
  | VIDE  -> emit "\\pspicture(0,0)(1,1)\n"
  end;

  (* met toutes les commandes *)
  List.iter pstricks img;

  (* pilogue *)
  emit "\\endpspicture\n\n\\bye\n"
;;
