(*******************************************************************************
 *                                                                             *
 *   Graphx : utilitaires de conversion                                        *
 *                                                                             *
 *******************************************************************************)

open Graph;;
open Printf;;

(* conversion d'un arc d'ellipse en courbes de bzier *)
let rec bezier_of_arc(fill,p,rx,ry,a1,a2) =
(*
 * pb : il n'y a pas  ma connaissance de fonction de trac d'un arc d'ellipse
 * en postscript. On pourrait utiliser le trac d'un arc de cercle en modifiant
 * la matrice de transformation, mais ceci conduit  un trait d'paisseur
 * ingale. Solution temporaire : un arc d'ellipse est approch par un ou
 * plusieurs arcs de Bzier, un si |a2-a1| <= Pi/2, plusieurs par division en
 * arcs d'ouverture infrieure  Pi/2 sinon.
 * Reste  trouver une approximation convenable d'un arc de cercle d'ouverture
 * alpha : (un arc de cercle car le pb est invariant par affinit) aprs
 * quelques calculs pnibles on trouve que pour que le milieu de l'arc de
 * Bzier concide avec le milieu de l'arc de cercle, il faut faut placer les
 * points de contrle  (4/3)*R*tan(alpha/4) sur les tangentes aux points
 * extrmits. Ce critre de concidence des milieux ne fournit probablement
 * pas la meilleure approximation uniforme d'un arc de cercle par un arc de
 * Bzier, toute personne dsireuse de se taper les calculs et trouver la
 * disposition optimale des points de contrle (et de me la communiquer) sera
 * dement remercie.
 *)

  (* vacuation des ouvertures ngatives ou trop grandes *)
  if a1 = a2          then [||]
  else if a1 > a2     then bezier_of_arc(fill,p,rx,ry,a2,a1)
  else if a2-a1 > 360 then bezier_of_arc(fill,p,rx,ry,a1,a2-360)
  else begin

    (* n = nb d'arcs de Bzier  produire           *)
    (* a = angle d'ouverture d'un arc en radians    *)
    (* co,si,t = lignes trigonomtriques associes *)
    let n = (a2-a1+89)/90 in
    let a = float(a2-a1) *. atan(1.0) /. 45.0 /. float(n) in
    let co = cos(a) and si = sin(a) and t = 4.0*.tan(a/.4.0)/.3.0 in

    (* rx,ry = rayons en flottant *)
    let rx = float(rx)  and ry = float(ry)   in

    (* M0,M1,M2,M3 = points de Bzier pour le cercle unit partant de a1 *)
    let a1 = float(a1) *. atan(1.0) /. 45.0          in
    let x0 = ref(cos a1)        and y0 = ref(sin a1)        in
    let x1 = ref(!x0 -. !y0*.t) and y1 = ref(!y0 +. !x0*.t) in
    let x3 = ref(cos(a1+.a))    and y3 = ref(sin(a1+.a))    in
    let x2 = ref(!x3 +. !y3*.t) and y2 = ref(!y3 -. !x3*.t) in

    (* tourne(x,y) fait tourner le point (x,y) de l'angle a *)
    let tourne(x,y) = let u = !x and v = !y in
      x := co*.u -. si*.v;
      y := si*.u +. co*.v
    in

    (* conversion flottant -> entier *)
    let point(x,y) = {x = p.x + truncate(!x*.rx);
                      y = p.y + truncate(!y*.ry)} in

    (* place les points *)
    let res = Array.create (3*n+1 + (if fill then 3 else 0)) (point(x0,y0)) in
    for i=1 to n do
      res.(3*i-2) <- point(x1,y1);
      res.(3*i-1) <- point(x2,y2);
      res.(3*i)   <- point(x3,y3);
      tourne(x1,y1);
      tourne(x2,y2);
      tourne(x3,y3)
    done;
    (* pour un arc plein, ajoute un segment vers le centre *)
    if fill then begin
      res.(3*n+1) <- res.(3*n);
      res.(3*n+2) <- p;
      res.(3*n+3) <- p
    end;
    res
  end
;;

(* conversion d'une commande en texte pour commentaire *)
(* [comm] est la chaine d'ouverture de commentaire *)
let comment chout comm =

  let emit format = fprintf chout format comm
  and emitv v =
   for i=0 to Array.length v - 1 do
     fprintf chout " %d %d" v.(i).x v.(i).y
   done;
   fprintf chout "\n"
  and s = function true -> "fill_" | false -> "draw_"
  in

function
| Set_color(c)      	  -> emit "%s set_color %d\n"       c
| Set_line_width(w) 	  -> emit "%s set_line_width %d\n"  w
| Set_font(f)       	  -> emit "%s set_font %s\n"        f
| Set_text_size(sz) 	  -> emit "%s set_text_size %d\n"   sz
| Poly(fill,v)      	  -> emit "%s %spoly"                    (s fill);   emitv v
| Bezier(fill,d,v)  	  -> emit "%s %sbezier %d"               (s fill) d; emitv v
| Arc(fill,p,rx,ry,a1,a2) -> emit "%s %sarc %d %d %d %d %d %d\n" (s fill) p.x p.y rx ry a1 a2
| Ellipse(fill,p,rx,ry)   -> emit "%s %sellipse %d %d %d %d\n"   (s fill) p.x p.y rx ry
| String(p,s)             -> emit "%s draw_text %d %d %s\n" p.x p.y (String.escaped s)
;;
